/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.2 (build: 2013-08-21 15:27:30 UTC)
 *  on 2013-08-23 at 21:06:51 UTC 
 */

package com.google.api.services.mirror;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Mirror (v1).
 *
 * <p>
 * API for interacting with Glass users via the timeline.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/glass" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link MirrorRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Mirror extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Google Mirror API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "mirror/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Mirror(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Mirror(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Contacts collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Mirror mirror = new Mirror(...);}
   *   {@code Mirror.Contacts.List request = mirror.contacts().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Contacts contacts() {
    return new Contacts();
  }

  /**
   * The "contacts" collection of methods.
   */
  public class Contacts {

    /**
     * Deletes a contact.
     *
     * Create a request for the method "contacts.delete".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the contact.
     * @return the request
     */
    public Delete delete(String id) throws java.io.IOException {
      Delete result = new Delete(id);
      initialize(result);
      return result;
    }

    public class Delete extends MirrorRequest<Void> {

      private static final String REST_PATH = "contacts/{id}";

      /**
       * Deletes a contact.
       *
       * Create a request for the method "contacts.delete".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the contact.
       * @since 1.13
       */
      protected Delete(String id) {
        super(Mirror.this, "DELETE", REST_PATH, null, Void.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** The ID of the contact. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the contact.
       */
      public String getId() {
        return id;
      }

      /** The ID of the contact. */
      public Delete setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Gets a single contact by ID.
     *
     * Create a request for the method "contacts.get".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the contact.
     * @return the request
     */
    public Get get(String id) throws java.io.IOException {
      Get result = new Get(id);
      initialize(result);
      return result;
    }

    public class Get extends MirrorRequest<com.google.api.services.mirror.model.Contact> {

      private static final String REST_PATH = "contacts/{id}";

      /**
       * Gets a single contact by ID.
       *
       * Create a request for the method "contacts.get".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the contact.
       * @since 1.13
       */
      protected Get(String id) {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.Contact.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** The ID of the contact. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the contact.
       */
      public String getId() {
        return id;
      }

      /** The ID of the contact. */
      public Get setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Inserts a new contact.
     *
     * Create a request for the method "contacts.insert".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.mirror.model.Contact}
     * @return the request
     */
    public Insert insert(com.google.api.services.mirror.model.Contact content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends MirrorRequest<com.google.api.services.mirror.model.Contact> {

      private static final String REST_PATH = "contacts";

      /**
       * Inserts a new contact.
       *
       * Create a request for the method "contacts.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.mirror.model.Contact}
       * @since 1.13
       */
      protected Insert(com.google.api.services.mirror.model.Contact content) {
        super(Mirror.this, "POST", REST_PATH, content, com.google.api.services.mirror.model.Contact.class);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

    }
    /**
     * Retrieves a list of contacts for the authenticated user.
     *
     * Create a request for the method "contacts.list".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends MirrorRequest<com.google.api.services.mirror.model.ContactsListResponse> {

      private static final String REST_PATH = "contacts";

      /**
       * Retrieves a list of contacts for the authenticated user.
       *
       * Create a request for the method "contacts.list".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.ContactsListResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

    }
    /**
     * Updates a contact in place. This method supports patch semantics.
     *
     * Create a request for the method "contacts.patch".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the contact.
     * @param content the {@link com.google.api.services.mirror.model.Contact}
     * @return the request
     */
    public Patch patch(String id, com.google.api.services.mirror.model.Contact content) throws java.io.IOException {
      Patch result = new Patch(id, content);
      initialize(result);
      return result;
    }

    public class Patch extends MirrorRequest<com.google.api.services.mirror.model.Contact> {

      private static final String REST_PATH = "contacts/{id}";

      /**
       * Updates a contact in place. This method supports patch semantics.
       *
       * Create a request for the method "contacts.patch".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the contact.
       * @param content the {@link com.google.api.services.mirror.model.Contact}
       * @since 1.13
       */
      protected Patch(String id, com.google.api.services.mirror.model.Contact content) {
        super(Mirror.this, "PATCH", REST_PATH, content, com.google.api.services.mirror.model.Contact.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** The ID of the contact. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the contact.
       */
      public String getId() {
        return id;
      }

      /** The ID of the contact. */
      public Patch setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Updates a contact in place.
     *
     * Create a request for the method "contacts.update".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the contact.
     * @param content the {@link com.google.api.services.mirror.model.Contact}
     * @return the request
     */
    public Update update(String id, com.google.api.services.mirror.model.Contact content) throws java.io.IOException {
      Update result = new Update(id, content);
      initialize(result);
      return result;
    }

    public class Update extends MirrorRequest<com.google.api.services.mirror.model.Contact> {

      private static final String REST_PATH = "contacts/{id}";

      /**
       * Updates a contact in place.
       *
       * Create a request for the method "contacts.update".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the contact.
       * @param content the {@link com.google.api.services.mirror.model.Contact}
       * @since 1.13
       */
      protected Update(String id, com.google.api.services.mirror.model.Contact content) {
        super(Mirror.this, "PUT", REST_PATH, content, com.google.api.services.mirror.model.Contact.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** The ID of the contact. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the contact.
       */
      public String getId() {
        return id;
      }

      /** The ID of the contact. */
      public Update setId(String id) {
        this.id = id;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Locations collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Mirror mirror = new Mirror(...);}
   *   {@code Mirror.Locations.List request = mirror.locations().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Locations locations() {
    return new Locations();
  }

  /**
   * The "locations" collection of methods.
   */
  public class Locations {

    /**
     * Gets a single location by ID.
     *
     * Create a request for the method "locations.get".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the location or latest for the last known location.
     * @return the request
     */
    public Get get(String id) throws java.io.IOException {
      Get result = new Get(id);
      initialize(result);
      return result;
    }

    public class Get extends MirrorRequest<com.google.api.services.mirror.model.Location> {

      private static final String REST_PATH = "locations/{id}";

      /**
       * Gets a single location by ID.
       *
       * Create a request for the method "locations.get".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the location or latest for the last known location.
       * @since 1.13
       */
      protected Get(String id) {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.Location.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** The ID of the location or latest for the last known location. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the location or latest for the last known location.
       */
      public String getId() {
        return id;
      }

      /** The ID of the location or latest for the last known location. */
      public Get setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Retrieves a list of locations for the user.
     *
     * Create a request for the method "locations.list".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends MirrorRequest<com.google.api.services.mirror.model.LocationsListResponse> {

      private static final String REST_PATH = "locations";

      /**
       * Retrieves a list of locations for the user.
       *
       * Create a request for the method "locations.list".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.LocationsListResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

    }

  }

  /**
   * An accessor for creating requests from the Subscriptions collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Mirror mirror = new Mirror(...);}
   *   {@code Mirror.Subscriptions.List request = mirror.subscriptions().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Subscriptions subscriptions() {
    return new Subscriptions();
  }

  /**
   * The "subscriptions" collection of methods.
   */
  public class Subscriptions {

    /**
     * Deletes a subscription.
     *
     * Create a request for the method "subscriptions.delete".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the subscription.
     * @return the request
     */
    public Delete delete(String id) throws java.io.IOException {
      Delete result = new Delete(id);
      initialize(result);
      return result;
    }

    public class Delete extends MirrorRequest<Void> {

      private static final String REST_PATH = "subscriptions/{id}";

      /**
       * Deletes a subscription.
       *
       * Create a request for the method "subscriptions.delete".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the subscription.
       * @since 1.13
       */
      protected Delete(String id) {
        super(Mirror.this, "DELETE", REST_PATH, null, Void.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** The ID of the subscription. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the subscription.
       */
      public String getId() {
        return id;
      }

      /** The ID of the subscription. */
      public Delete setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Creates a new subscription.
     *
     * Create a request for the method "subscriptions.insert".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.mirror.model.Subscription}
     * @return the request
     */
    public Insert insert(com.google.api.services.mirror.model.Subscription content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends MirrorRequest<com.google.api.services.mirror.model.Subscription> {

      private static final String REST_PATH = "subscriptions";

      /**
       * Creates a new subscription.
       *
       * Create a request for the method "subscriptions.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.mirror.model.Subscription}
       * @since 1.13
       */
      protected Insert(com.google.api.services.mirror.model.Subscription content) {
        super(Mirror.this, "POST", REST_PATH, content, com.google.api.services.mirror.model.Subscription.class);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

    }
    /**
     * Retrieves a list of subscriptions for the authenticated user and service.
     *
     * Create a request for the method "subscriptions.list".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends MirrorRequest<com.google.api.services.mirror.model.SubscriptionsListResponse> {

      private static final String REST_PATH = "subscriptions";

      /**
       * Retrieves a list of subscriptions for the authenticated user and service.
       *
       * Create a request for the method "subscriptions.list".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.SubscriptionsListResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

    }
    /**
     * Updates an existing subscription in place.
     *
     * Create a request for the method "subscriptions.update".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the subscription.
     * @param content the {@link com.google.api.services.mirror.model.Subscription}
     * @return the request
     */
    public Update update(String id, com.google.api.services.mirror.model.Subscription content) throws java.io.IOException {
      Update result = new Update(id, content);
      initialize(result);
      return result;
    }

    public class Update extends MirrorRequest<com.google.api.services.mirror.model.Subscription> {

      private static final String REST_PATH = "subscriptions/{id}";

      /**
       * Updates an existing subscription in place.
       *
       * Create a request for the method "subscriptions.update".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the subscription.
       * @param content the {@link com.google.api.services.mirror.model.Subscription}
       * @since 1.13
       */
      protected Update(String id, com.google.api.services.mirror.model.Subscription content) {
        super(Mirror.this, "PUT", REST_PATH, content, com.google.api.services.mirror.model.Subscription.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** The ID of the subscription. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the subscription.
       */
      public String getId() {
        return id;
      }

      /** The ID of the subscription. */
      public Update setId(String id) {
        this.id = id;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Timeline collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Mirror mirror = new Mirror(...);}
   *   {@code Mirror.Timeline.List request = mirror.timeline().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Timeline timeline() {
    return new Timeline();
  }

  /**
   * The "timeline" collection of methods.
   */
  public class Timeline {

    /**
     * Deletes a timeline item.
     *
     * Create a request for the method "timeline.delete".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the timeline item.
     * @return the request
     */
    public Delete delete(String id) throws java.io.IOException {
      Delete result = new Delete(id);
      initialize(result);
      return result;
    }

    public class Delete extends MirrorRequest<Void> {

      private static final String REST_PATH = "timeline/{id}";

      /**
       * Deletes a timeline item.
       *
       * Create a request for the method "timeline.delete".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the timeline item.
       * @since 1.13
       */
      protected Delete(String id) {
        super(Mirror.this, "DELETE", REST_PATH, null, Void.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** The ID of the timeline item. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the timeline item.
       */
      public String getId() {
        return id;
      }

      /** The ID of the timeline item. */
      public Delete setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Gets a single timeline item by ID.
     *
     * Create a request for the method "timeline.get".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the timeline item.
     * @return the request
     */
    public Get get(String id) throws java.io.IOException {
      Get result = new Get(id);
      initialize(result);
      return result;
    }

    public class Get extends MirrorRequest<com.google.api.services.mirror.model.TimelineItem> {

      private static final String REST_PATH = "timeline/{id}";

      /**
       * Gets a single timeline item by ID.
       *
       * Create a request for the method "timeline.get".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the timeline item.
       * @since 1.13
       */
      protected Get(String id) {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.TimelineItem.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      initializeMediaDownload();
      }

      /** Upgrade warning: in prior version 1.8 this method was called {@code download}. */
      @Override
      public void executeMediaAndDownloadTo(java.io.OutputStream outputStream) throws java.io.IOException {
        super.executeMediaAndDownloadTo(outputStream);
      }

      @Override
      public java.io.InputStream executeMediaAsInputStream() throws java.io.IOException {
        return super.executeMediaAsInputStream();
      }

      @Override
      public com.google.api.client.http.HttpResponse executeMedia() throws java.io.IOException {
        return super.executeMedia();
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** The ID of the timeline item. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the timeline item.
       */
      public String getId() {
        return id;
      }

      /** The ID of the timeline item. */
      public Get setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Inserts a new item into the timeline.
     *
     * Create a request for the method "timeline.insert".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
     * @return the request
     */
    public Insert insert(com.google.api.services.mirror.model.TimelineItem content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    /**
     * Inserts a new item into the timeline.
     *
     * Create a request for the method "timeline.insert".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * <p>
     * This method should be used for uploading media content.
     * </p>
     *
     *
     * @param content the {@link com.google.api.services.mirror.model.TimelineItem} media metadata or {@code null} if none
     * @param mediaContent The media HTTP content or {@code null} if none.
     * @return the request
     * @throws java.io.IOException if the initialization of the request fails
     */
    public Insert insert(com.google.api.services.mirror.model.TimelineItem content,
        com.google.api.client.http.AbstractInputStreamContent mediaContent) throws java.io.IOException {
      Insert result = new Insert(content, mediaContent);
      initialize(result);
      return result;
    }

    public class Insert extends MirrorRequest<com.google.api.services.mirror.model.TimelineItem> {

      private static final String REST_PATH = "timeline";

      /**
       * Inserts a new item into the timeline.
       *
       * Create a request for the method "timeline.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
       * @since 1.13
       */
      protected Insert(com.google.api.services.mirror.model.TimelineItem content) {
        super(Mirror.this, "POST", REST_PATH, content, com.google.api.services.mirror.model.TimelineItem.class);
      }

      /**
       * Inserts a new item into the timeline.
       *
       * Create a request for the method "timeline.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * <p>
       * This constructor should be used for uploading media content.
       * </p>
       *
       *
       * @param content the {@link com.google.api.services.mirror.model.TimelineItem} media metadata or {@code null} if none
       * @param mediaContent The media HTTP content or {@code null} if none.
       * @since 1.13
       */
      protected Insert(com.google.api.services.mirror.model.TimelineItem content,
          com.google.api.client.http.AbstractInputStreamContent mediaContent) {
        super(Mirror.this, "POST", "/upload/" + getServicePath() + REST_PATH, content, com.google.api.services.mirror.model.TimelineItem.class);
        initializeMediaUpload(mediaContent);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

    }
    /**
     * Retrieves a list of timeline items for the authenticated user.
     *
     * Create a request for the method "timeline.list".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends MirrorRequest<com.google.api.services.mirror.model.TimelineListResponse> {

      private static final String REST_PATH = "timeline";

      /**
       * Retrieves a list of timeline items for the authenticated user.
       *
       * Create a request for the method "timeline.list".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.TimelineListResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Controls the order in which timeline items are returned. */
      @com.google.api.client.util.Key
      private String orderBy;

      /** Controls the order in which timeline items are returned.
       */
      public String getOrderBy() {
        return orderBy;
      }

      /** Controls the order in which timeline items are returned. */
      public List setOrderBy(String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /** If true, tombstone records for deleted items will be returned. */
      @com.google.api.client.util.Key
      private Boolean includeDeleted;

      /** If true, tombstone records for deleted items will be returned.
       */
      public Boolean getIncludeDeleted() {
        return includeDeleted;
      }

      /** If true, tombstone records for deleted items will be returned. */
      public List setIncludeDeleted(Boolean includeDeleted) {
        this.includeDeleted = includeDeleted;
        return this;
      }

      /** The maximum number of items to include in the response, used for paging. */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** The maximum number of items to include in the response, used for paging.
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** The maximum number of items to include in the response, used for paging. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token for the page of results to return. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token for the page of results to return.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token for the page of results to return. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** If provided, only items with the given sourceItemId will be returned. */
      @com.google.api.client.util.Key
      private String sourceItemId;

      /** If provided, only items with the given sourceItemId will be returned.
       */
      public String getSourceItemId() {
        return sourceItemId;
      }

      /** If provided, only items with the given sourceItemId will be returned. */
      public List setSourceItemId(String sourceItemId) {
        this.sourceItemId = sourceItemId;
        return this;
      }

      /** If true, only pinned items will be returned. */
      @com.google.api.client.util.Key
      private Boolean pinnedOnly;

      /** If true, only pinned items will be returned.
       */
      public Boolean getPinnedOnly() {
        return pinnedOnly;
      }

      /** If true, only pinned items will be returned. */
      public List setPinnedOnly(Boolean pinnedOnly) {
        this.pinnedOnly = pinnedOnly;
        return this;
      }

      /** If provided, only items with the given bundleId will be returned. */
      @com.google.api.client.util.Key
      private String bundleId;

      /** If provided, only items with the given bundleId will be returned.
       */
      public String getBundleId() {
        return bundleId;
      }

      /** If provided, only items with the given bundleId will be returned. */
      public List setBundleId(String bundleId) {
        this.bundleId = bundleId;
        return this;
      }

    }
    /**
     * Updates a timeline item in place. This method supports patch semantics.
     *
     * Create a request for the method "timeline.patch".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the timeline item.
     * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
     * @return the request
     */
    public Patch patch(String id, com.google.api.services.mirror.model.TimelineItem content) throws java.io.IOException {
      Patch result = new Patch(id, content);
      initialize(result);
      return result;
    }

    public class Patch extends MirrorRequest<com.google.api.services.mirror.model.TimelineItem> {

      private static final String REST_PATH = "timeline/{id}";

      /**
       * Updates a timeline item in place. This method supports patch semantics.
       *
       * Create a request for the method "timeline.patch".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the timeline item.
       * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
       * @since 1.13
       */
      protected Patch(String id, com.google.api.services.mirror.model.TimelineItem content) {
        super(Mirror.this, "PATCH", REST_PATH, content, com.google.api.services.mirror.model.TimelineItem.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** The ID of the timeline item. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the timeline item.
       */
      public String getId() {
        return id;
      }

      /** The ID of the timeline item. */
      public Patch setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Updates a timeline item in place.
     *
     * Create a request for the method "timeline.update".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param id The ID of the timeline item.
     * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
     * @return the request
     */
    public Update update(String id, com.google.api.services.mirror.model.TimelineItem content) throws java.io.IOException {
      Update result = new Update(id, content);
      initialize(result);
      return result;
    }

    /**
     * Updates a timeline item in place.
     *
     * Create a request for the method "timeline.update".
     *
     * This request holds the parameters needed by the the mirror server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * <p>
     * This method should be used for uploading media content.
     * </p>
     *
     * @param id The ID of the timeline item.
     * @param content the {@link com.google.api.services.mirror.model.TimelineItem} media metadata or {@code null} if none
     * @param mediaContent The media HTTP content or {@code null} if none.
     * @return the request
     * @throws java.io.IOException if the initialization of the request fails
     */
    public Update update(String id, com.google.api.services.mirror.model.TimelineItem content,
        com.google.api.client.http.AbstractInputStreamContent mediaContent) throws java.io.IOException {
      Update result = new Update(id, content, mediaContent);
      initialize(result);
      return result;
    }

    public class Update extends MirrorRequest<com.google.api.services.mirror.model.TimelineItem> {

      private static final String REST_PATH = "timeline/{id}";

      /**
       * Updates a timeline item in place.
       *
       * Create a request for the method "timeline.update".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param id The ID of the timeline item.
       * @param content the {@link com.google.api.services.mirror.model.TimelineItem}
       * @since 1.13
       */
      protected Update(String id, com.google.api.services.mirror.model.TimelineItem content) {
        super(Mirror.this, "PUT", REST_PATH, content, com.google.api.services.mirror.model.TimelineItem.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      /**
       * Updates a timeline item in place.
       *
       * Create a request for the method "timeline.update".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * <p>
       * This constructor should be used for uploading media content.
       * </p>
       *
       * @param id The ID of the timeline item.
       * @param content the {@link com.google.api.services.mirror.model.TimelineItem} media metadata or {@code null} if none
       * @param mediaContent The media HTTP content or {@code null} if none.
       * @since 1.13
       */
      protected Update(String id, com.google.api.services.mirror.model.TimelineItem content,
          com.google.api.client.http.AbstractInputStreamContent mediaContent) {
        super(Mirror.this, "PUT", "/upload/" + getServicePath() + REST_PATH, content, com.google.api.services.mirror.model.TimelineItem.class);
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
        initializeMediaUpload(mediaContent);
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** The ID of the timeline item. */
      @com.google.api.client.util.Key
      private String id;

      /** The ID of the timeline item.
       */
      public String getId() {
        return id;
      }

      /** The ID of the timeline item. */
      public Update setId(String id) {
        this.id = id;
        return this;
      }

    }

    /**
     * An accessor for creating requests from the Attachments collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Mirror mirror = new Mirror(...);}
     *   {@code Mirror.Attachments.List request = mirror.attachments().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Attachments attachments() {
      return new Attachments();
    }

    /**
     * The "attachments" collection of methods.
     */
    public class Attachments {

      /**
       * Deletes an attachment from a timeline item.
       *
       * Create a request for the method "attachments.delete".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param itemId The ID of the timeline item the attachment belongs to.
       * @param attachmentId The ID of the attachment.
       * @return the request
       */
      public Delete delete(String itemId, String attachmentId) throws java.io.IOException {
        Delete result = new Delete(itemId, attachmentId);
        initialize(result);
        return result;
      }

      public class Delete extends MirrorRequest<Void> {

        private static final String REST_PATH = "timeline/{itemId}/attachments/{attachmentId}";

        /**
         * Deletes an attachment from a timeline item.
         *
         * Create a request for the method "attachments.delete".
         *
         * This request holds the parameters needed by the the mirror server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param itemId The ID of the timeline item the attachment belongs to.
         * @param attachmentId The ID of the attachment.
         * @since 1.13
         */
        protected Delete(String itemId, String attachmentId) {
          super(Mirror.this, "DELETE", REST_PATH, null, Void.class);
          this.itemId = Preconditions.checkNotNull(itemId, "Required parameter itemId must be specified.");
          this.attachmentId = Preconditions.checkNotNull(attachmentId, "Required parameter attachmentId must be specified.");
        }

        @Override
        public Delete setAlt(String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** The ID of the timeline item the attachment belongs to. */
        @com.google.api.client.util.Key
        private String itemId;

        /** The ID of the timeline item the attachment belongs to.
         */
        public String getItemId() {
          return itemId;
        }

        /** The ID of the timeline item the attachment belongs to. */
        public Delete setItemId(String itemId) {
          this.itemId = itemId;
          return this;
        }

        /** The ID of the attachment. */
        @com.google.api.client.util.Key
        private String attachmentId;

        /** The ID of the attachment.
         */
        public String getAttachmentId() {
          return attachmentId;
        }

        /** The ID of the attachment. */
        public Delete setAttachmentId(String attachmentId) {
          this.attachmentId = attachmentId;
          return this;
        }

      }
      /**
       * Retrieves an attachment on a timeline item by item ID and attachment ID.
       *
       * Create a request for the method "attachments.get".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation.
       *
       * @param itemId The ID of the timeline item the attachment belongs to.
       * @param attachmentId The ID of the attachment.
       * @return the request
       */
      public Get get(String itemId, String attachmentId) throws java.io.IOException {
        Get result = new Get(itemId, attachmentId);
        initialize(result);
        return result;
      }

      public class Get extends MirrorRequest<com.google.api.services.mirror.model.Attachment> {

        private static final String REST_PATH = "timeline/{itemId}/attachments/{attachmentId}";

        /**
         * Retrieves an attachment on a timeline item by item ID and attachment ID.
         *
         * Create a request for the method "attachments.get".
         *
         * This request holds the parameters needed by the the mirror server.  After setting any optional
         * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
         * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param itemId The ID of the timeline item the attachment belongs to.
         * @param attachmentId The ID of the attachment.
         * @since 1.13
         */
        protected Get(String itemId, String attachmentId) {
          super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.Attachment.class);
          this.itemId = Preconditions.checkNotNull(itemId, "Required parameter itemId must be specified.");
          this.attachmentId = Preconditions.checkNotNull(attachmentId, "Required parameter attachmentId must be specified.");
        initializeMediaDownload();
        }

        /** Upgrade warning: in prior version 1.8 this method was called {@code download}. */
        @Override
        public void executeMediaAndDownloadTo(java.io.OutputStream outputStream) throws java.io.IOException {
          super.executeMediaAndDownloadTo(outputStream);
        }

        @Override
        public java.io.InputStream executeMediaAsInputStream() throws java.io.IOException {
          return super.executeMediaAsInputStream();
        }

        @Override
        public com.google.api.client.http.HttpResponse executeMedia() throws java.io.IOException {
          return super.executeMedia();
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public Get setAlt(String alt) {
          return (Get) super.setAlt(alt);
        }

        @Override
        public Get setFields(String fields) {
          return (Get) super.setFields(fields);
        }

        @Override
        public Get setKey(String key) {
          return (Get) super.setKey(key);
        }

        @Override
        public Get setOauthToken(String oauthToken) {
          return (Get) super.setOauthToken(oauthToken);
        }

        @Override
        public Get setPrettyPrint(Boolean prettyPrint) {
          return (Get) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Get setQuotaUser(String quotaUser) {
          return (Get) super.setQuotaUser(quotaUser);
        }

        @Override
        public Get setUserIp(String userIp) {
          return (Get) super.setUserIp(userIp);
        }

        /** The ID of the timeline item the attachment belongs to. */
        @com.google.api.client.util.Key
        private String itemId;

        /** The ID of the timeline item the attachment belongs to.
         */
        public String getItemId() {
          return itemId;
        }

        /** The ID of the timeline item the attachment belongs to. */
        public Get setItemId(String itemId) {
          this.itemId = itemId;
          return this;
        }

        /** The ID of the attachment. */
        @com.google.api.client.util.Key
        private String attachmentId;

        /** The ID of the attachment.
         */
        public String getAttachmentId() {
          return attachmentId;
        }

        /** The ID of the attachment. */
        public Get setAttachmentId(String attachmentId) {
          this.attachmentId = attachmentId;
          return this;
        }

      }
      /**
       * Adds a new attachment to a timeline item.
       *
       * Create a request for the method "attachments.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * @param itemId The ID of the timeline item the attachment belongs to.
       * @return the request
       */
      public Insert insert(String itemId) throws java.io.IOException {
        Insert result = new Insert(itemId);
        initialize(result);
        return result;
      }

      /**
       * Adds a new attachment to a timeline item.
       *
       * Create a request for the method "attachments.insert".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * <p>
       * This method should be used for uploading media content.
       * </p>
       *
       * @param itemId The ID of the timeline item the attachment belongs to.
       * @param mediaContent The media HTTP content or {@code null} if none.
       * @return the request
       * @throws java.io.IOException if the initialization of the request fails
       */
      public Insert insert(String itemId,
          com.google.api.client.http.AbstractInputStreamContent mediaContent) throws java.io.IOException {
        Insert result = new Insert(itemId, mediaContent);
        initialize(result);
        return result;
      }

      public class Insert extends MirrorRequest<com.google.api.services.mirror.model.Attachment> {

        private static final String REST_PATH = "timeline/{itemId}/attachments";

        /**
         * Adds a new attachment to a timeline item.
         *
         * Create a request for the method "attachments.insert".
         *
         * This request holds the parameters needed by the the mirror server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param itemId The ID of the timeline item the attachment belongs to.
         * @since 1.13
         */
        protected Insert(String itemId) {
          super(Mirror.this, "POST", REST_PATH, null, com.google.api.services.mirror.model.Attachment.class);
          this.itemId = Preconditions.checkNotNull(itemId, "Required parameter itemId must be specified.");
        }

        /**
         * Adds a new attachment to a timeline item.
         *
         * Create a request for the method "attachments.insert".
         *
         * This request holds the parameters needed by the the mirror server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * <p>
         * This constructor should be used for uploading media content.
         * </p>
         *
         * @param itemId The ID of the timeline item the attachment belongs to.
         * @param mediaContent The media HTTP content or {@code null} if none.
         * @since 1.13
         */
        protected Insert(String itemId,
            com.google.api.client.http.AbstractInputStreamContent mediaContent) {
          super(Mirror.this, "POST", "/upload/" + getServicePath() + REST_PATH, null, com.google.api.services.mirror.model.Attachment.class);
          this.itemId = Preconditions.checkNotNull(itemId, "Required parameter itemId must be specified.");
          initializeMediaUpload(mediaContent);
        }

        @Override
        public Insert setAlt(String alt) {
          return (Insert) super.setAlt(alt);
        }

        @Override
        public Insert setFields(String fields) {
          return (Insert) super.setFields(fields);
        }

        @Override
        public Insert setKey(String key) {
          return (Insert) super.setKey(key);
        }

        @Override
        public Insert setOauthToken(String oauthToken) {
          return (Insert) super.setOauthToken(oauthToken);
        }

        @Override
        public Insert setPrettyPrint(Boolean prettyPrint) {
          return (Insert) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Insert setQuotaUser(String quotaUser) {
          return (Insert) super.setQuotaUser(quotaUser);
        }

        @Override
        public Insert setUserIp(String userIp) {
          return (Insert) super.setUserIp(userIp);
        }

        /** The ID of the timeline item the attachment belongs to. */
        @com.google.api.client.util.Key
        private String itemId;

        /** The ID of the timeline item the attachment belongs to.
         */
        public String getItemId() {
          return itemId;
        }

        /** The ID of the timeline item the attachment belongs to. */
        public Insert setItemId(String itemId) {
          this.itemId = itemId;
          return this;
        }

      }
      /**
       * Returns a list of attachments for a timeline item.
       *
       * Create a request for the method "attachments.list".
       *
       * This request holds the parameters needed by the the mirror server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation.
       *
       * @param itemId The ID of the timeline item whose attachments should be listed.
       * @return the request
       */
      public List list(String itemId) throws java.io.IOException {
        List result = new List(itemId);
        initialize(result);
        return result;
      }

      public class List extends MirrorRequest<com.google.api.services.mirror.model.AttachmentsListResponse> {

        private static final String REST_PATH = "timeline/{itemId}/attachments";

        /**
         * Returns a list of attachments for a timeline item.
         *
         * Create a request for the method "attachments.list".
         *
         * This request holds the parameters needed by the the mirror server.  After setting any optional
         * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
         * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param itemId The ID of the timeline item whose attachments should be listed.
         * @since 1.13
         */
        protected List(String itemId) {
          super(Mirror.this, "GET", REST_PATH, null, com.google.api.services.mirror.model.AttachmentsListResponse.class);
          this.itemId = Preconditions.checkNotNull(itemId, "Required parameter itemId must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public List setAlt(String alt) {
          return (List) super.setAlt(alt);
        }

        @Override
        public List setFields(String fields) {
          return (List) super.setFields(fields);
        }

        @Override
        public List setKey(String key) {
          return (List) super.setKey(key);
        }

        @Override
        public List setOauthToken(String oauthToken) {
          return (List) super.setOauthToken(oauthToken);
        }

        @Override
        public List setPrettyPrint(Boolean prettyPrint) {
          return (List) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public List setQuotaUser(String quotaUser) {
          return (List) super.setQuotaUser(quotaUser);
        }

        @Override
        public List setUserIp(String userIp) {
          return (List) super.setUserIp(userIp);
        }

        /** The ID of the timeline item whose attachments should be listed. */
        @com.google.api.client.util.Key
        private String itemId;

        /** The ID of the timeline item whose attachments should be listed.
         */
        public String getItemId() {
          return itemId;
        }

        /** The ID of the timeline item whose attachments should be listed. */
        public List setItemId(String itemId) {
          this.itemId = itemId;
          return this;
        }

      }

    }
  }

  /**
   * Builder for {@link Mirror}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Mirror}. */
    @Override
    public Mirror build() {
      return new Mirror(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link MirrorRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setMirrorRequestInitializer(
        MirrorRequestInitializer mirrorRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(mirrorRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
